
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

module Paginator
  extend ActiveSupport::Concern

  def paginate(model)
    total_count model
    @values = model.limit(limit).offset(offset) if model.respond_to?('limit')
    return params_actions(@values) if ( @values )
    return model[offset..limit]
  end

  private

    def limit
      return params[:limit].to_i if !params[:limit].blank?
      return params[:results_per_page].to_i if !params[:results_per_page].blank?
      return 10
    end


    def offset
      return params[:offset].to_i if !params[:offset].blank?
      return params[:page].to_i*params[:results_per_page].to_i if !params[:page].blank? && !params[:results_per_page].blank?
      return 0
    end


    def params_actions(values)

      # filtering
      #   name, author (LearningObject), city (Institution), privacy (Collections)
      if ( !params[:filter].blank? )
        hash_params = JSON.parse params[:filter]
        hash_params.each do |key, value|
          values = values.where("#{key} ILIKE ?", "%#{value}%")
        end
      end

      # ordering
      #   name, id, author, score, email, city
      sort_p = params[:sort] if !params[:sort].blank?
      values = values.sort { |a,b| (a[sort_p].is_a? Integer)? a[sort_p] <=> b[sort_p] : a[sort_p].to_s.downcase <=> b[sort_p].to_s.downcase } if sort_p and params[:order] == 'ASC'
      values = values.sort { |a,b| (a[sort_p].is_a? Integer)? -(a[sort_p] <=> b[sort_p]) : -(a[sort_p].to_s.downcase <=> b[sort_p].to_s.downcase) } if sort_p and params[:order] == 'DESC'

      return values
    end

    def total_count(model)
      headers['X-Total-Count'] = model.count
    end

end
